<?php
namespace Pinpoint\BannerManager\Setup;

use Magento\Framework\DB\Ddl\Table;
use Magento\Framework\Setup\UpgradeSchemaInterface;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\SchemaSetupInterface;
/**
 * Class UpgradeSchema
 * @package Pinpoint\BannerManager\Setup
 * @codeCoverageIgnore
 */
class UpgradeSchema implements UpgradeSchemaInterface
{
    /**
     * {@inheritdoc}
     */
    public function upgrade(SchemaSetupInterface $setup, ModuleContextInterface $context)
    {
        $setup->startSetup();

        if (version_compare($context->getVersion(), '0.0.2', '<')) {
            $this->createBannerTable($setup);
        }

        if (version_compare($context->getVersion(), '0.0.3', '<')) {
            $this->createBannerBlockTable($setup);
        }

        if (version_compare($context->getVersion(), '0.0.4', '<')) {
            $this->addBlockTextSizeColumn($setup);
        }

        if (version_compare($context->getVersion(), '0.0.5', '<')) {
            $this->addBlockHideOnMobileColumn($setup);
        }

        if (version_compare($context->getVersion(), '0.0.6', '<')) {
            $this->addBlockImagePositionColumn($setup);
        }

        if (version_compare($context->getVersion(), '0.0.7', '<')) {
            $this->addBannerDeviceType($setup);
        }

        $setup->endSetup();
    }

    /**
     * @param SchemaSetupInterface $setup
     */
    protected function createBannerTable(SchemaSetupInterface $setup)
    {
        $installer = $setup;
        $installer->startSetup();

        $tableName = $installer->getTable('pinpoint_banners');
        // Check if the table already exists
        if ($installer->getConnection()->isTableExists($tableName) != true) {
            $table = $installer->getConnection()
                ->newTable($tableName)
                ->addColumn(
                    'id',
                    Table::TYPE_INTEGER,
                    null,
                    [
                        'identity' => true,
                        'unsigned' => true,
                        'nullable' => false,
                        'primary' => true
                    ],
                    'ID'
                )
                ->addColumn(
                    'banner_name',
                    Table::TYPE_TEXT,
                    null,
                    ['nullable' => false],
                    'Banner Name'
                )
                ->addColumn(
                    'banner_content',
                    Table::TYPE_TEXT,
                    null,
                    ['nullable' => false],
                    'Banner Content'
                )
                ->addColumn(
                    'enabled',
                    Table::TYPE_SMALLINT,
                    null,
                    ['nullable' => false, 'default' => 1],
                    'Enabled'
                )
                ->setComment('Pinpoint Banner Manager Banners')
                ->setOption('type', 'InnoDB')
                ->setOption('charset', 'utf8');
            $installer->getConnection()->createTable($table);
        }

        $installer->endSetup();
    }

    /**
     * @param SchemaSetupInterface $setup
     */
    protected function createBannerBlockTable(SchemaSetupInterface $setup)
    {
        $installer = $setup;
        $installer->startSetup();

        $tableName = $installer->getTable('pinpoint_bannermanager_bannerblock');
        // Check if the table already exists
        if ($installer->getConnection()->isTableExists($tableName) != true) {
            $table = $installer->getConnection()
                ->newTable($tableName)
                ->addColumn(
                    'id',
                    Table::TYPE_INTEGER,
                    null,
                    [
                        'identity' => true,
                        'unsigned' => true,
                        'nullable' => false,
                        'primary' => true
                    ],
                    'ID'
                )
                ->addColumn(
                    'banner_id',
                    Table::TYPE_SMALLINT,
                    null,
                    ['nullable' => false],
                    'Parent Banner ID'
                )
                ->addColumn(
                    'block_ref',
                    Table::TYPE_TEXT,
                    null,
                    ['nullable' => false],
                    'Block Reference'
                )
                ->addColumn(
                    'image',
                    Table::TYPE_TEXT,
                    null,
                    ['nullable' => false],
                    'Banner image'
                )
                ->addColumn(
                    'text',
                    Table::TYPE_TEXT,
                    null,
                    ['nullable' => false],
                    'Banner Text'
                )
                ->addColumn(
                    'text_colour',
                    Table::TYPE_TEXT,
                    null,
                    ['nullable' => false],
                    'Banner text colour'
                )
                ->addColumn(
                    'text_position',
                    Table::TYPE_TEXT,
                    null,
                    ['nullable' => false],
                    'Banner text position'
                )
                ->addColumn(
                    'link_text',
                    Table::TYPE_TEXT,
                    null,
                    ['nullable' => false],
                    'Banner link text'
                )
                ->addColumn(
                    'link_url',
                    Table::TYPE_TEXT,
                    null,
                    ['nullable' => false],
                    'Banner link url'
                )
                ->addColumn(
                    'link_target',
                    Table::TYPE_TEXT,
                    null,
                    ['nullable' => false],
                    'Banner link target'
                )
                ->addColumn(
                    'enabled',
                    Table::TYPE_SMALLINT,
                    null,
                    ['nullable' => false, 'default' => 1],
                    'Enabled'
                )
                ->setComment('Pinpoint Banner Manager Banner Blocks')
                ->setOption('type', 'InnoDB')
                ->setOption('charset', 'utf8');
            $installer->getConnection()->createTable($table);
        }

        $installer->endSetup();
    }

    /**
     * @param SchemaSetupInterface $setup
     */
    protected function addBlockTextSizeColumn(SchemaSetupInterface $setup)
    {
        $installer = $setup;
        $installer->startSetup();

        $installer->getConnection()->addColumn(
            $installer->getTable('pinpoint_bannermanager_bannerblock'),
            'block_text_size',
            [
                'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                'nullable' => true,
                'comment' => 'Block Text Size'
            ]
        );

        $installer->endSetup();
    }

    /**
     * @param SchemaSetupInterface $setup
     */
    protected function addBlockHideOnMobileColumn(SchemaSetupInterface $setup)
    {
        $installer = $setup;
        $installer->startSetup();

        $installer->getConnection()->addColumn(
            $installer->getTable('pinpoint_bannermanager_banners'),
            'hide_on_mobile',
            [
                'type' => \Magento\Framework\DB\Ddl\Table::TYPE_SMALLINT,
                'nullable' => false,
                'default' => 0,
                'comment' => 'Block Text Size'
            ]
        );

        $installer->endSetup();
    }

    /**
     * @param SchemaSetupInterface $setup
     */
    protected function addBlockImagePositionColumn(SchemaSetupInterface $setup)
    {
        $installer = $setup;
        $installer->startSetup();

        $installer->getConnection()->addColumn(
            $installer->getTable('pinpoint_bannermanager_bannerblock'),
            'block_image_location',
            [
                'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                'nullable' => true,
                'comment' => 'Block Image Position'
            ]
        );

        $installer->endSetup();
    }

    /**
     * @param SchemaSetupInterface $setup
     */
    protected function addBannerDeviceType(SchemaSetupInterface $setup)
    {
        $installer = $setup;
        $installer->startSetup();

        $installer->getConnection()->addColumn(
            $installer->getTable('pinpoint_banners'),
            'device_type',
            [
                'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                'nullable' => false,
                'comment' => 'device banner will be shown on',
                'default' => 'all',
                'length' => 255
            ]
        );

        $installer->endSetup();
    }
}
